import sys

from bgesdk.client import API
from bgesdk.error import APIError
from bgesdk.management import constants
from bgesdk.management.command import BaseCommand
from bgesdk.management.utils import (
    config_get,
    get_active_project,
    output,
    output_json,
    read_config
)
from bgesdk.models import ModelEncoder


DEFAULT_OAUTH2_SECTION = constants.DEFAULT_OAUTH2_SECTION
DEFAULT_TOKEN_SECTION = constants.DEFAULT_TOKEN_SECTION


class Command(BaseCommand):

    order = 9
    help='聚合组学数据（目前仅支持聚合数据流中符合平台设定 JSONPath 规则的数值型数据）。'

    def add_arguments(self, parser):
        parser.add_argument(
            'data_element_id',
            type=str,
            help='数据元编号。'
        )
        parser.add_argument(
            'time_dimension',
            choices=(
                'year', 'quarter', 'month', 'week', 'day', 'minute', 'second'
            ),
            default='day',
            type=str,
            help='子聚合的时间维度。'
        )
        parser.add_argument(
            '--start_time',
            required=True,
            type=str,
            help='数据流生成时间的起始时间。'
        )
        parser.add_argument(
            '--end_time',
            type=str,
            help='数据流生成时间的结束时间，为空时默认取当前时间。'
        )
        parser.add_argument(
            '-b',
            '--biosample_id',
            type=str,
            help='生物样品编号。'
        )
        parser.add_argument(
            '-i',
            '--interval',
            type=int,
            default=1,
            help='聚合时间维度间隔。'
        )
        parser.add_argument(
            '--periods',
            type=int,
            default=100,
            help='聚合时间维度返回数。'
        )
        parser.add_argument(
            '-t',
            '--access_token',
            type=str,
            help='访问令牌，为空时默认使用 bge token 命令获取到的 access_token。'
        )

    def handler(self, args):
        access_token = args.access_token
        project = get_active_project()
        oauth2_section = DEFAULT_OAUTH2_SECTION
        token_section = DEFAULT_TOKEN_SECTION
        config = read_config(project)
        if not access_token:
            access_token = config_get(config.get, token_section, 'access_token')
        endpoint = config_get(config.get, oauth2_section, 'endpoint')
        api = API(access_token, endpoint=endpoint, timeout=18.)
        try:
            result = api.aggregate_omics_data(
                args.data_element_id,
                args.time_dimension,
                args.start_time,
                end_time=args.end_time,
                biosample_id=args.biosample_id,
                interval=args.interval,
                periods=args.periods
            )
        except APIError as e:
            output('[red]请求失败：[/red]')
            output_json(e.result)
            sys.exit(1)
        output('[green]请求成功：[/green]')
        output_json(
            result,
            cls=ModelEncoder
        )



#-*- coding: utf-8 -*-

from bgesdk.client import OAuth2
from six.moves.urllib.parse import parse_qsl, urlparse

import logging
import os
import pytest
import requests
import six


ENDPOINT = os.environ.get('ENDPOINT')
CLIENT_ID = os.environ.get('CLIENT_ID')
CLIENT_SECRET = os.environ.get('CLIENT_SECRET')
REDIRECT_URI = os.environ.get('REDIRECT_URI')
AUTHORIZATION_EXTRA_QUERY = os.environ.get('AUTHORIZATION_EXTRA_QUERY')
SELF_BIOSAMPLE_ID = os.environ.get('SELF_BIOSAMPLE_ID')
SELF_META_BIOSAMPLE_ID = os.environ.get('SELF_META_BIOSAMPLE_ID')
OTHER_BIOSAMPLE_ID = os.environ.get('OTHER_BIOSAMPLE_ID')


@pytest.fixture(scope='session')
def redirect_uri():
    return REDIRECT_URI


@pytest.fixture(scope='session')
def authorization_url(oauth2, redirect_uri):
    authorization_url = oauth2.get_authorization_url(redirect_uri)
    return '&'.join((authorization_url, AUTHORIZATION_EXTRA_QUERY))


@pytest.fixture(scope='session')
def authorization_code(authorization_url):
    r = requests.get(authorization_url)
    assert r.status_code == 200
    query = dict(parse_qsl(urlparse(r.url).query))
    code = query.get('code')
    assert isinstance(code, six.string_types)
    return code


@pytest.fixture(scope='session')
def logger():
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    return logger


@pytest.fixture(scope='session')
def oauth2():
    return OAuth2(CLIENT_ID, CLIENT_SECRET, endpoint=ENDPOINT, max_retries=3)


@pytest.fixture(scope='session')
def access_token(oauth2, authorization_code, redirect_uri):
    token = oauth2.exchange_authorization_code(
        authorization_code,
        redirect_uri
    )
    return token.access_token


@pytest.fixture(scope='session')
def api(oauth2, access_token):
    return oauth2.get_api(access_token)


@pytest.fixture(scope='session')
def self_biosample_id():
    """本人的样品编号"""
    return SELF_BIOSAMPLE_ID


@pytest.fixture(scope='session')
def self_meta_biosample_id():
    """本人的 Meta 样品编号"""
    return SELF_META_BIOSAMPLE_ID


@pytest.fixture(scope='session')
def other_biosample_id():
    """其他人的样品编号"""
    return OTHER_BIOSAMPLE_ID



import argparse
import os
import sys

from posixpath import exists, splitext
from rich.prompt import Prompt
from rich.table import Table

from bgesdk.management import constants
from bgesdk.management.command import BaseCommand
from bgesdk.management.utils import (
    config_get,
    confirm,
    console,
    get_active_project,
    get_config_dir,
    get_config_parser,
    get_config_path,
    output,
    output_file,
    secure_str,
)


class Command(BaseCommand):

    order = 1
    help = '配置、新增、显示、删除 BGE 开放平台的 OAuth2 配置，仅支持客户端模式。'
    project_help = '全局配置项目名。'

    def add_arguments(self, parser):
        sub_ps = parser.add_subparsers(
            help='新增、删除、显示 OAuth2 配置',
            required=False
        )
        add_p = sub_ps.add_parser(
            'add',
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
            help='新增 BGE 开放平台配置。'
        )
        add_p.add_argument(
            'project',
            help=self.project_help
        )
        add_p.set_defaults(method=self.add_project, parser=add_p)
        remove_p = sub_ps.add_parser(
            'remove',
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
            help='配置、显示、删除 BGE 开放平台的 OAuth2 全局配置，仅支持客户端模式。'
        )
        remove_p.add_argument(
            'project',
            help=self.project_help
        )
        remove_p.set_defaults(method=self.remove_project, parser=remove_p)
        show_p = sub_ps.add_parser(
            'show',
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
            help='打印显示全局配置信息。'
        )
        show_p.add_argument(
            'project',
            nargs='?',
            help=self.project_help
        )
        show_p.set_defaults(method=self.show_project, parser=show_p)
        list_p = sub_ps.add_parser(
            'list',
            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
            help='显示全部已配置的项目。'
        )
        list_p.set_defaults(method=list_projects, parser=list_p)

    def handler(self, args):
        project = get_active_project()
        output('当前正在配置 {}：\n'.format(project))
        config_path = get_config_path(project, check_exists=False)
        self._new_project(config_path)

    def add_project(self, args):
        project = args.project.lower()
        config_path = get_config_path(project, check_exists=False)
        if exists(config_path):
            output('[red]配置 {} 已存在'.format(project))
            sys.exit(1)
        self._new_project(config_path)

    def _get_config_value(self, config_parser, key, type_):
        oauth2_section = constants.DEFAULT_OAUTH2_SECTION
        if 'str' == type_:
            saved_value = config_get(
                config_parser.get, oauth2_section, key)
        elif 'int' == type_:
            saved_value = config_get(
                config_parser.getint, oauth2_section, key)
        elif 'bool' == type_:
            saved_value = config_get(
                config_parser.getboolean, oauth2_section, key)
        else:
            raise ValueError('invalid type: {}'.format(type_))
        return saved_value

    def _new_project(self, config_path):
        config_parser = get_config_parser(config_path)
        oauth2_section = constants.DEFAULT_OAUTH2_SECTION
        if oauth2_section not in config_parser.sections():
            config_parser.add_section(oauth2_section)
        for key, conf in constants.CLIENT_CREDENTIALS_CONFIGS:
            type_ = conf['type']
            saved_value = self._get_config_value(config_parser, key, type_)
            secure = conf.get('secure')
            description = conf.get('description', '')
            value = saved_value or conf.get('default', '')
            show_value = value
            if secure and value:
                show_value = secure_str(value)
            input_value = Prompt.ask(
                '请输入{} {} ([bold cyan]{}[/bold cyan])'.format(
                    description, key, show_value
                ),
                show_default=False,
                default=value
            )
            if input_value:
                config_parser.set(oauth2_section, key, input_value)
            elif saved_value is None:
                if conf.get('default') is not None:
                    config_parser.set(
                        oauth2_section, key, conf.get('default'))
                else:
                    config_parser.set(oauth2_section, key, '')
        with open(config_path, 'w') as config_file:
            config_parser.write(config_file)
        output('')
        output('[green]配置已保存至：[/green]{}'.format(config_path))

    def remove_project(self, args):
        project = args.project.lower()
        activate_project = get_active_project()
        if activate_project == project:
            output(
                '[red]无法删除正在使用的配置，请先使用 bge workon 切换至其他项目[/red]'
            )
            sys.exit(1)
        config_path = get_config_path(project)
        if not confirm(prompt='确认删除配置项目 {}？'.format(project)):
            output('已取消删除')
            sys.exit(0)
        try:
            os.unlink(config_path)
        except (IOError, OSError):
            pass
        output('[green]成功删除配置项目[/green] {}'.format(project))

    def show_project(self, args):
        project = args.project
        if not project:
            project = get_active_project()
        project = project.lower()
        config_path = get_config_path(project)
        title = 'BGE 开放平台 Python SDK 配置文件'
        output_file(config_path, title=title, subtitle=config_path)


def list_projects(args=None):
    active_project = get_active_project()
    config_dir = get_config_dir()
    projects = []
    for filename in os.listdir(config_dir):
        name, ext = splitext(filename)
        if ext != '.ini':
            continue
        projects.append(name)
    projects.sort()
    table = Table(
        title='通过 bge workon <NAME> 切换生效配置',
        expand=True,
        show_header=True,
        header_style="magenta"
    )
    table.add_column(
        "项目",
        justify="center",
        style="dim"
    )
    table.add_column("使用中", justify="center")
    if active_project in projects:
        table.add_row(
            active_project,
            'Y',
            style="green",
        )
    for project in projects:
        if project == active_project:
            continue
        table.add_row(project, '-')
    console.print(table)



#-*- coding: utf-8 -*-

from bgesdk.fs import FileItem

import pytest


class TestFS:

    def test_file_item(self):
        inst = FileItem('hello.txt', 'Hello world')
        assert isinstance(inst, FileItem)


import sys

from bgesdk.client import API
from bgesdk.error import APIError
from bgesdk.management import constants
from bgesdk.management.command import BaseCommand
from bgesdk.management.utils import (
    config_get,
    get_active_project,
    output,
    output_json,
    read_config
)
from bgesdk.models import ModelEncoder


DEFAULT_OAUTH2_SECTION = constants.DEFAULT_OAUTH2_SECTION
DEFAULT_TOKEN_SECTION = constants.DEFAULT_TOKEN_SECTION


class Command(BaseCommand):

    order = 7
    help='请求数据项。'

    def add_arguments(self, parser):
        parser.add_argument(
            'biosample_id',
            type=str,
            help='生物样品编号。'
        )
        parser.add_argument(
            'namespace',
            type=str,
            help='命名空间'
        )
        parser.add_argument(
            '--data_element_ids',
            type=str,
            help='数据元编号，与 collection_id 互斥。'
        )
        parser.add_argument(
            '--collection_id',
            type=str,
            help='数据集编号，与 data_element_ids 互斥。'
        )
        parser.add_argument(
            '-p',
            '--next_page',
            type=int,
            help='要获取的页码。'
        )
        parser.add_argument(
            '-t',
            '--access_token',
            type=str,
            help='访问令牌，为空时默认使用 bge token 命令获取到的 access_token。'
        )

    def handler(self, args):
        access_token = args.access_token
        namespace = args.namespace
        biosample_id = args.biosample_id
        project = get_active_project()
        oauth2_section = DEFAULT_OAUTH2_SECTION
        token_section = DEFAULT_TOKEN_SECTION
        config = read_config(project)
        if not access_token:
            access_token = config_get(config.get, token_section, 'access_token')
        endpoint = config_get(config.get, oauth2_section, 'endpoint')
        api = API(access_token, endpoint=endpoint, timeout=18.)
        try:
            result = api.get_data_items(
                namespace,
                biosample_id,
                collection_id=args.collection_id,
                data_element_ids=args.data_element_ids,
                next_page=args.next_page
            )
        except APIError as e:
            output('[red]请求失败：[/red]')
            output_json(e.result)
            sys.exit(1)
        output('[green]请求成功：[/green]')
        output_json(
            result,
            cls=ModelEncoder
        )



import os
import sys


from rich.table import Table
from posixpath import join, isfile

from bgesdk.client import API
from bgesdk.error import APIError
from bgesdk.management import constants
from bgesdk.management.command import BaseCommand
from bgesdk.management.utils import (
    config_get,
    console,
    get_active_project,
    output,
    output_json,
    read_config
)


DEFAULT_OAUTH2_SECTION = constants.DEFAULT_OAUTH2_SECTION
DEFAULT_TOKEN_SECTION = constants.DEFAULT_TOKEN_SECTION


class Command(BaseCommand):

    order = 11
    help='上传目录下文件（不递归上传子目录中文件）。'

    def add_arguments(self, parser):
        parser.add_argument(
            'dirpath',
            type=str,
            help='要上传的文件夹。'
        )
        parser.add_argument(
            '-i',
            '--cmk_id',
            help='阿里云 KMS 服务密钥 ID，提供时代表使用加密方式上传文件。'
        )
        parser.add_argument(
            '-t',
            '--access_token',
            type=str,
            help='访问令牌，为空时默认使用 bge token 命令获取到的 access_token。'
        )

    def handler(self, args):
        access_token = args.access_token
        project = get_active_project()
        oauth2_section = DEFAULT_OAUTH2_SECTION
        token_section = DEFAULT_TOKEN_SECTION
        config = read_config(project)
        if not access_token:
            access_token = config_get(config.get, token_section, 'access_token')
        endpoint = config_get(config.get, oauth2_section, 'endpoint')
        api = API(access_token, endpoint=endpoint, timeout=18.)
        dirpath = args.dirpath
        cmk_id = args.cmk_id
        files = []
        for filename in os.listdir(dirpath):
            filepath = join(dirpath, filename)
            if isfile(filepath):
                files.append(filepath)
        if not files:
            output('[red]文件夹中没有可上传的文件[/red]')
            sys.exit(1)
        message = '正在上传目录 {}'.format(dirpath)
        with console.status(message, spinner='earth'):
            try:
                object_names = api.upload_dir(dirpath, cmk_id=cmk_id)
            except APIError as e:
                output('[red]请求失败：[/red]')
                output_json(e.result)
                sys.exit(1)
            output('[green]文件上传成功：[/green]')
            table = Table(
                title="文件列表",
                expand=True,
                show_header=True,
                header_style="magenta"
            )
            table.add_column("序号", style="cyan", no_wrap=True)
            table.add_column("存储位置", style="magenta", overflow='fold')
            for i, object_name in enumerate(object_names, 1):
                table.add_row(str(i), object_name)
            console.print(table)



#!/usr/bin/env python

import sys

if sys.version_info.major <= 2:
    from six.moves import reload_module

    reload_module(sys)
    sys.setdefaultencoding("UTF-8")

import setuptools

with open('requirements.txt') as f:
    install_requires = [line.strip() for line in f]

setuptools.setup(
    install_requires=install_requires
)


#-*- coding: utf-8 -*-

import json
import os
import pkgutil
import platform
import six
import sys

from os.path import expanduser
from posixpath import join, exists, abspath
from rich.console import Console
from rich.panel import Panel
from rich.prompt import Confirm
from rich.syntax import Syntax
from six.moves import configparser

from . import __path__, constants

if six.PY2:
    ConfigParser = configparser.SafeConfigParser
else:
    ConfigParser = configparser.ConfigParser

NoSectionError = configparser.NoSectionError
NoOptionError = configparser.NoOptionError

SYS_STR = platform.system().lower()

console = Console()


class CNConfirm(Confirm):

    validate_error_message = "[prompt.invalid]请输入 Y 或 N"


def get_home():
    return abspath('.')


def get_user_home():
    return expanduser("~")


def get_config_dir():
    user_home = get_user_home()
    bge_home = join(user_home, '.bge')
    if not exists(bge_home):
        os.makedirs(bge_home)
    return bge_home


def get_config_path(project, check_exists=True):
    config_dir = get_config_dir()
    config_path = join(config_dir, '{}.ini'.format(project))
    if not exists(config_path) and check_exists:
        output(
            '{} 不存在，请运行 bge config 或 bge config add 命令初始化项目配置'
                .format(project)
        )
        sys.exit(1)
    return config_path


def confirm(prompt=None):
    if prompt is None:
        prompt = '确认'
    return CNConfirm.ask(prompt)


def find_commands(commands_dir):
    """查找子命令列表"""
    return [
        name for _, name, is_pkg in pkgutil.iter_modules(
            [commands_dir])
    ]


def get_commands():
    """获取全部子命令"""
    return find_commands(join(__path__[0], 'commands'))


def get_active_path():
    config_dir = get_config_dir()
    return join(config_dir, 'activate_project')


def get_active_project():
    active_path = get_active_path()
    project = constants.DEFAULT_PROJECT
    if exists(active_path):
        with open(active_path, 'r') as fp:
            project = fp.read()
    else:
        with open(active_path, 'w') as fp:
            fp.write(project)
    return project


def generate_container_name(model_id):
    return '{}{}'.format(constants.MODEL_CONTAINER_PREFIX, model_id)


def config_get(method, section_name, key, default=None):
    try:
        return method(section_name, key)
    except (NoOptionError, NoSectionError):
        return default


def secure_str(s):
    if len(s) > 6:
        return s.replace(s[3:-3], '*' * 6)
    if len(s) >= 1:
        return s[0] + '*' * 6
    return s


def read_config(project):
    config_path = get_config_path(project)
    config = ConfigParser()
    config.read(config_path)
    return config


def get_config_parser(path):
    config_parser = ConfigParser()
    config_parser.read(path)
    return config_parser


def output_json(data, cls=None):
    syntax = Syntax(
        json.dumps(
            data,
            indent=4,
            sort_keys=True,
            ensure_ascii=False,
            cls=cls
        ),
        lexer='JSON',
        line_numbers=False,
        word_wrap=True,
        theme='monokai'
    )
    console.print(syntax)


def output_file(filepath, title='', subtitle=''):
    syntax = Syntax.from_path(
        filepath,
        line_numbers=True,
        word_wrap=True,
        theme='monokai'
    )
    panel = Panel(syntax, title=title, subtitle=subtitle)
    console.print(panel)


def output_panel(content, title='', subtitle='', lexer='python'):
    syntax = Syntax(
        content,
        lexer=lexer,
        line_numbers=True,
        word_wrap=True,
        theme='monokai'
    )
    panel = Panel(syntax, title=title, subtitle=subtitle)
    console.print(panel)


def output_syntax(content,
                  lexer='python',
                  line_numbers=True,
                  theme='monokai'):
    syntax = Syntax(
        content,
        lexer=lexer,
        line_numbers=line_numbers,
        word_wrap=True,
        theme=theme
    )
    console.print(syntax)


def output(*args, sep=' '):
    args = map(lambda x: str(x), args)
    content = '{}'.format(sep).join(args)
    console.print(content)


def get_sys_user():
    if SYS_STR == 'windows':
        user = '1000:1000'
    else:
        import pwd
        pwd_info = pwd.getpwuid(os.getuid())
        uid = pwd_info.pw_uid
        gid = pwd_info.pw_gid
        user = '{}:{}'.format(uid, gid)
    return user



import sys
import webbrowser

from rich.prompt import Prompt

from bgesdk.client import OAuth2
from bgesdk.error import APIError
from bgesdk.management import constants
from bgesdk.management.command import BaseCommand
from bgesdk.management.utils import (
    config_get,
    get_active_project,
    get_config_parser,
    get_config_path,
    output,
    output_json,
    output_syntax
)


class Command(BaseCommand):

    order = 3
    help='授权码模式登录并获取授权令牌。'

    def add_arguments(self, parser):
        parser.add_argument(
            'redirect_uri',
            help='授权完成后的回调地址。'
        )
        parser.add_argument(
            '--state',
            help='额外信息，原样作为参数返回给回调地址。'
        )
        parser.add_argument(
            '--scopes',
            help='权限标识'
        )

    def handler(self, args):
        project = get_active_project()
        config_path = get_config_path(project)
        config_parser = get_config_parser(config_path)
        oauth2_section = constants.DEFAULT_OAUTH2_SECTION
        client_id = config_get(config_parser.get, oauth2_section, 'client_id')
        client_secret = config_get(
            config_parser.get, oauth2_section, 'client_secret')
        endpoint = config_get(config_parser.get, oauth2_section, 'endpoint')
        redirect_uri = args.redirect_uri
        oauth2 = OAuth2(
            client_id, client_secret, endpoint=endpoint, timeout=60.)
        authorization_url = oauth2.get_authorization_url(
            redirect_uri, state=args.state, scopes=args.scopes
        )
        output('授权页面地址为：{}'.format(authorization_url))
        output('浏览器启动中...')
        webbrowser.open(authorization_url, new=1, autoraise=False)
        output('请在浏览器登录并完成授权，复制跳转后页面链接的 code 参数并关闭浏览器。\n')
        while True:
            code = Prompt.ask('请输入浏览器回调地址携带的参数 [cyan]code[/cyan]')
            code = code.strip()
            if code:
                break
            output('[red]参数 code 不能为空[/red]')
        try:
            token_result = oauth2.exchange_authorization_code(
                code, redirect_uri=redirect_uri
            )
        except APIError as e:
            output('[red]令牌获取出错：[/red]')
            output_json(e.result)
            sys.exit(1)
        self._write_token_config(project, token_result)
        output('[green]令牌内容如下[/green]')
        content = []
        for key in [
                'access_token', 'token_type', 'expires_in', 'scope',
                'refresh_token']:
            content.append('{} = {}'.format(key, token_result[key]))
        output_syntax('\n'.join(content), lexer='INI')

    def _write_token_config(self, project, token_result):
        access_token = token_result['access_token']
        token_type = token_result['token_type']
        expires_in = str(token_result['expires_in'])
        scope = token_result['scope']
        refresh_token = token_result['refresh_token']
        config_path = get_config_path(project)
        config_parser = get_config_parser(config_path)
        section_name = constants.DEFAULT_TOKEN_SECTION
        if section_name not in config_parser.sections():
            config_parser.add_section(section_name)
        config_parser.set(section_name, 'access_token', access_token)
        config_parser.set(section_name, 'token_type', token_type)
        config_parser.set(section_name, 'expires_in', expires_in)
        config_parser.set(section_name, 'scope', scope)
        config_parser.set(section_name, 'refresh_token', refresh_token)
        with open(config_path, 'w') as config_file:
            config_parser.write(config_file)
        output('[green]令牌已保存至：[/green]{}'.format(config_path))
